#include "entity.h"


Entity::Entity(float x, float y, float w, float h) :
	bbox(x, y - 0.5*h, w, h)
{
	pos[0] = x;
	pos[1] = y;
}

Entity::~Entity()
{
}

void
Entity::getPosition(float& x, float& y) const
{
	x = pos[0];
	y = pos[1];
}

void
Entity::setPosition(float  x, float  y)
{
	pos[0] = x;
	pos[1] = y;

	bbox.setCenter(x, y - 0.5*bbox.getHeight());
}

BBox&
Entity::getBBox()
{
	return bbox;
}

bool
Entity::addSprite(int id, Sprite* sprite)
{
	pair<int, Sprite*> item;
	item.first  = id;
	item.second = sprite;

	pair<map<int, Sprite*>::iterator, bool> result;
	result = sprites.insert(item);

	//
	// it'd be nice to 'grow' the bbox here based
	// on frame dimensions as each sprite is added;
	// might give a better bbox fix
	//

	return result.second;
}

Sprite*
Entity::removeSprite(int id)
{
	map<int, Sprite*>::iterator iter = sprites.find(id);

	if( iter != sprites.end() )
	{
		Sprite* sprite = (*iter).second;
		sprites.erase(iter);
		return sprite;
	}

	return 0;
}

Sprite*
Entity::getSprite(int id) const
{
	map<int, Sprite*>::const_iterator iter = sprites.find(id);

	if( iter != sprites.end() )
		return (*iter).second;

	return 0;
}

int
Entity::getNumSprites() const
{
	return sprites.size();
}

void
Entity::update(double dt)
{
	Sprite* sprite;

	map<int, Sprite*>::const_iterator iter;
	for( iter = sprites.begin(); iter != sprites.end(); iter++ )
	{
		sprite = (*iter).second;
		sprite->IncrementTimer(dt);
	}
}

void
Entity::draw(bool drawBBox) const
{
	Sprite* sprite;

	map<int, Sprite*>::const_iterator iter;
	for( iter = sprites.begin(); iter != sprites.end(); iter++ )
	{
		sprite = (*iter).second;
		sprite->SetPos(pos[0], pos[1]);
		sprite->Draw();
	}

	if( drawBBox )
	{
		bbox.draw();
	}
}
