#ifndef __SPELLSOURCE_H__
#define __SPELLSOURCE_H__

#include "enum.h"
#include "entity.h"
#include "spellPoints.h"

#include "SpellShot.h"


enum eSpellReaction { eNone, eShotFizzle, eSourceFizzle };

// THIS IS AN ABSTRACT CLASS!
// Must define type()
//
class SpellSource : public Entity
{
public:

	//
	// need to add alpha based on remaining lifetime
	// make entity update/draw virtual since derived
	// classes may want to do additional stuff
	//

	SpellSource(float x, float y);
	virtual ~SpellSource();

	static  SpellSource* createSpellSource(eSpellSources type, float x, float y, bool useDefault = true, int w = 0, int s = 0);

	virtual eSpellSources type() const = 0;

	// Affect shot with this source
	virtual eSpellReaction affectShot(SpellShot& s) const;

	float getLifetime() const;
	void  setLifetime(float t);
	float decLifetime(float dt);
	void  resetLifetime();

	bool  isDead() const;

	SpellPoints& getWizardSP();
	SpellPoints& getShotSP();

protected:

	SpellPoints dWizard;
	SpellPoints dSpellShot;

private:

	float lifetime;
};


class FireSource : public SpellSource
{
public:

	FireSource(float x, float y, int w = DEF_SS_WIZ_F, int s = DEF_SS_SHOT_F) :
		SpellSource(x, y)
	{
		dWizard.setFirePts(w);
		dSpellShot.setFirePts(s);
	}

	virtual ~FireSource() { }

	virtual eSpellSources type() const { return eFire; }

	virtual eSpellReaction affectShot(SpellShot& s) const;
};

class WaterSource : public SpellSource
{
public:

	WaterSource(float x, float y, int w = DEF_SS_WIZ_W, int s = DEF_SS_SHOT_W) :
		SpellSource(x, y)
	{
		dWizard.setWaterPts(w);
		dSpellShot.setWaterPts(s);
	}

	virtual ~WaterSource() { }

	virtual eSpellSources type() const { return eWater; }

	virtual eSpellReaction affectShot(SpellShot& s) const;
};

class EarthSource : public SpellSource
{
public:

	EarthSource(float x, float y, int w = DEF_SS_WIZ_E, int s = DEF_SS_SHOT_E) :
		SpellSource(x, y)
	{
		dWizard.setEarthPts(w);
		dSpellShot.setEarthPts(s);
	}

	virtual ~EarthSource() { }

	virtual eSpellSources type() const { return eEarth; }

	virtual eSpellReaction affectShot(SpellShot& s) const ;
};

class ElecSource : public SpellSource
{
public:

	ElecSource(float x, float y, int w = DEF_SS_WIZ_L, int s = DEF_SS_SHOT_L) :
		SpellSource(x, y)
	{
		dWizard.setElecPts(w);
		dSpellShot.setElecPts(s);
	}

	virtual ~ElecSource() { }

	virtual eSpellSources type() const { return eElec; }

	virtual eSpellReaction affectShot(SpellShot& s) const;
};

#endif // __SPELLSOURCE_H__